# ANSI coloring for terminal output messages
from prettier_prints.ansi_styling import STYLING_OPTIONS
import re


class PrettierPrints:
    def __init__(self, style: str = None):
        self.style = style

    # Custom message with color options rather than a set color output
    def out(self, msg: str = None, style: str = None, json_out: dict = {}):
        if msg is None and json_out != {}:
            msg = json_out.get('msg')

        if not msg:
            raise Exception("A message is required")

        formatted_msg = f"{STYLING_OPTIONS['pref']}"

        if style is None and json_out != {}:
            style = json_out.get('style')

        if style is not None:
            formatted_msg = self.get_styling_info(styling=style, formatted_string=formatted_msg)

        elif self.style is not None:
            formatted_msg = self.get_styling_info(styling=self.style, formatted_string=formatted_msg)

        formatted_msg += f"{msg}{STYLING_OPTIONS['reset']}"
        return formatted_msg

    def json(self, json_obj, style: str = None):
        if json_obj == {} or json_obj is None:
            raise Exception("Must be a valid JSON object")

        json_style = self.strip_styling_for_json(style)

        print('{')
        for key in json_obj.keys():
            formatted_string = f'{STYLING_OPTIONS["pref"]}'
            if type(json_obj[key]) == dict:
                dict_formatted_string = self.get_styling_info(styling=json_style['dict'], formatted_string=formatted_string)
                print(f"{dict_formatted_string}{key}{STYLING_OPTIONS['reset']}: {str('{')}")
                for inner_key in json_obj[key].keys():
                    print(f"{' ' * 2}{dict_formatted_string}{inner_key} {STYLING_OPTIONS['reset']}: {json_obj[key][inner_key]},")
                print(' },')

            elif type(json_obj[key]) == list:
                list_formatted_string = self.get_styling_info(styling=json_style['list'], formatted_string=formatted_string)
                print(f"{list_formatted_string}{key}{STYLING_OPTIONS['reset']}: {str('[')}")
                for item in json_obj[key]:
                    if type(item) == dict:
                        dict_formatted_string = self.get_styling_info(styling=json_style['dict'], formatted_string=formatted_string)
                        print(' ' * 3 + '{')
                        for inner_key in item.keys():
                            print(f"{' ' * 4}{dict_formatted_string}{inner_key} {STYLING_OPTIONS['reset']}: {item[inner_key]},")
                        print(' ' * 3 + '},')
                    else:
                        string_formatted_string = self.get_styling_info(styling=json_style['string'], formatted_string=formatted_string)
                        print(f"{' ' * 2}{string_formatted_string}{item}{STYLING_OPTIONS['reset']}")
                print(' ],')

            else:
                string_formatted_string = self.get_styling_info(styling=json_style['string'], formatted_string=formatted_string)
                print(f"{string_formatted_string}{key}{STYLING_OPTIONS['reset']}: {json_obj[key]},")

        print('}')

    def strip_styling_for_json(self, styling):
        return_styling = {
            'list': 'red',
            'dict': 'magenta',
            'string': 'green'
        }
        if styling is not None:
            if re.search('string=(.*?)', styling):
                if re.search('string=(.*?)&', styling):
                    return_styling['string'] = re.search('string=(.*?)&', styling).group(1)
                elif re.search('&string=(.*?)', styling):
                    return_styling['string'] = re.search('&string=(.*?)$', styling).group(1)

            if re.search('list=(.*?)', styling):
                if re.search('list=(.*?)&', styling):
                    return_styling['list'] = re.search('list=(.*?)&', styling).group(1)
                elif re.search('&list=(.*?)', styling):
                    return_styling['list'] = re.search('&list=(.*?)$', styling).group(1)

            if re.search('dict=(.*?)', styling):
                if re.search('dict=(.*?)&', styling):
                    return_styling['dict'] = re.search('dict=(.*?)&', styling).group(1)
                elif re.search('&dict=(.*?)', styling):
                    return_styling['dict'] = re.search('&dict=(.*?)$', styling).group(1)

        return return_styling

    def get_styling_info(self, styling, formatted_string):
        color = STYLING_OPTIONS["standard_colors"]["white"]
        formatted_style = ''
        specified_styles = styling.split(';')

        for styles in specified_styles:
            if STYLING_OPTIONS["standard_colors"].get(styles):
                color = STYLING_OPTIONS["standard_colors"].get(styles)
            elif STYLING_OPTIONS["bright_colors"].get(styles):
                color = STYLING_OPTIONS["bright_colors"].get(styles)
            else:
                if STYLING_OPTIONS.get(styles):
                    formatted_style += STYLING_OPTIONS.get(styles)

        formatted_string += f'{color}{formatted_style}'
        return formatted_string


if __name__ == '__main__':
    pp = PrettierPrints()
    pp.json(json_obj={'test': 'cool', 'test_two': 'cool_two', 'dict_check': {'test': 'hello'},
                      'list_check': [
                          'test',
                          'test two',
                          {
                              'test': 'hello',
                              'test_two': 'bloop'
                          },
                          {
                              'test': 'hello',
                              'test_two': 'bloop'
                          }
                      ]}, style='list=blue;underline&dict=red;bold&string=green;')

    # print(pp.strip_styling_for_json('string=red;underline&list=blue&dict=green;underline'))


# ANSI coloring for terminal output messages
# Old v1 version of the prettier prints module
STYLING_OPTIONS = {
    "pref": "\033[",
    "reset": "\033[0m",
    # Colors
    "standard_colors": {
        "black": "30m",
        "red": "31m",
        "green": "32m",
        "yellow": "33m",
        "blue": "34m",
        "magenta": "35m",
        "cyan": "36m",
        "white": "37m"
    },
    # Bright Colors
    "bright_colors": {
        "bright_red": "31;1m",
        "bright_green": "32;1m",
        "bright_yellow": "33;1m",
        "bright_blue": "34;1m",
        "bright_magenta": "35;1m",
        "bright_cyan": "36;1m",
        "bright_white": "37;1m",
    },
    # Styling
    "bold": "\033[1m",
    "underline": "\033[4m",
    "highlight": "\033[7m",
}


def _pref() -> str:
    return STYLING_OPTIONS['pref']


def _reset() -> str:
    return STYLING_OPTIONS['reset']


class Decorators:
    def __init__(self, message: str = ''):
        self.output_message = message

    def bold(self) -> 'Decorators':
        self.output_message = f"{STYLING_OPTIONS['bold']}{self.output_message}{_reset()}"
        return self

    def underline(self) -> 'Decorators':
        self.output_message = f"{STYLING_OPTIONS['underline']}{self.output_message}{_reset()}"
        return self

    def highlight(self) -> 'Decorators':
        self.output_message = f"{STYLING_OPTIONS['highlight']}{self.output_message}{_reset()}"
        return self


class Colors:
    def __init__(self, message: str = ''):
        self.output_message = message
    
    def red(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['red']}{self.output_message}{_reset()}"
        return self

    def green(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['green']}{self.output_message}{_reset()}"
        return self

    def yellow(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['yellow']}{self.output_message}{_reset()}"
        return self

    def blue(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['blue']}{self.output_message}{_reset()}"
        return self
    
    def magenta(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['magenta']}{self.output_message}{_reset()}"
        return self

    def cyan(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['cyan']}{self.output_message}{_reset()}"
        return self

    def white(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['standard_colors']['white']}{self.output_message}{_reset()}"
        return self

    def bright_red(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_red']}{self.output_message}{_reset()}"
        return self

    def bright_green(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_green']}{self.output_message}{_reset()}"
        return self

    def bright_yellow(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_yellow']}{self.output_message}{_reset()}"
        return self

    def bright_blue(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_blue']}{self.output_message}{_reset()}"
        return self

    def bright_magenta(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_magenta']}{self.output_message}{_reset()}"
        return self

    def bright_cyan(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_cyan']}{self.output_message}{_reset()}"
        return self

    def bright_white(self) -> 'Colors':
        self.output_message = f"{_pref()}{STYLING_OPTIONS['bright_colors']['bright_white']}{self.output_message}{_reset()}"
        return self


from prettier_prints.ansi_styling import Colors, Decorators


class Output(Colors, Decorators):
    def __init__(self):
        self.output_message: str = ''
        super().__init__(self.output_message)

    def __str__(self):
        return f"{self.output_message}"

    def msg(self, message: str):
        self.output_message: str = message
        return self

    # return the output message
    def out(self) -> str:
        return f'{self.output_message}'




from setuptools import setup, find_packages

from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

project_urls = {
    "Code Repo": "https://github.com/PhantomLeak/prettier_prints",
    "Bug Tracker": "https://github.com/PhantomLeak/prettier_prints/issues"
}

setup(
    name='prettier_prints',
    packages=find_packages(include=['prettier_prints']),
    version='3.0.0',
    description='Lightweight library for prettier terminal outputs and debugging similar to chalk for Javascript',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Dylan Stocking',
    author_email='phantomleak@protonmail.com',
    license='MIT',
    setup_requires=['pytest-runner'],
    tests_require=['pytest==7.4.0'],
    test_suite='tests',
    classifiers=["License :: OSI Approved :: MIT License",
                 "Programming Language :: Python",
                 "Programming Language :: Python :: 3",
                 "Topic :: Software Development",
                 "Operating System :: Microsoft :: Windows",
                 "Operating System :: Unix",
                 "Operating System :: MacOS",
                 ],
    keywords=["prints", "chalk", "output"],
    project_urls=project_urls,
)
